<?php


namespace ADELANTE\PayBySquare\Helper;


use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Magento\Framework\Exception\AlreadyExistsException;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\OrderRepository;
use Psr\Log\LoggerInterface;

class CreateQRCode
{

    const API_URL = 'https://app.bysquare.com/api/generateQR';

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var Data
     */
    protected $data;

    /**
     * @var Client
     */
    protected $client;

    /**
     * @var OrderRepository
     */
    protected $orderRepository;

    /**
     * CreateQRCode constructor.
     *
     * @param LoggerInterface $loggerInterface
     * @param OrderRepository $orderRepository
     * @param Data $data
     * @param Client $client
     */
    public function __construct(
        LoggerInterface $loggerInterface,
        Data $data,
        Client $client,
        OrderRepository $orderRepository
    ) {
        $this->logger = $loggerInterface;
        $this->data = $data;
        $this->client = $client;
        $this->orderRepository = $orderRepository;
    }

    /**
     * @param $order
     */
    public function execute(&$order)
    {
        if (!in_array($order->getOrderCurrencyCode(), $this->data->getActiveFormat())) {
            return;
        }

        $response = $this->fetchQrCode($order);
        if ($response['success']) {
            $order->setData('adelante_pay_by_square_code', $response['value']);
            $this->orderRepository->save($order);
        }
    }

    /**
     * @param Order $order
     * @return array
     */
    protected function fetchQrCode(Order $order)
    {
        if (!$this->isBankTranferPayment($order)) {
            return ['success' => 'false', 'message' => 'Order payment method have to be "Bank Transfer"'];
        }

        // Create xml
        $xml = $this->createXML($order);

        // Send request to api
        $client = $this->client;
        try {
            $result = $client->request('POST', self::API_URL, [
                'headers' => [
                    'content-type' => 'text/xml',
                ],
                'body' => $xml,
                'http_errors' => false
            ]);
        } catch (GuzzleException $e) {
            $this->logger->error('Paybysquare exception: ' . $e->getMessage());
            return ['success' => false, 'message' => $e->getMessage()];
        }

        // Something is wrong
        if ($result->getStatusCode() !== 200) {
            $message = simplexml_load_string($result->getBody()->getContents());

            $this->logger->error('Paybysquare: ' . $message->Message);
            return ['success' => false, 'message' => $message->Message];
        }

        $code = simplexml_load_string($result->getBody()->getContents());

        $value = '';
        if ($order->getOrderCurrencyCode() === 'EUR') {
            $value = $code->PayBySquare;
        } else if ($order->getOrderCurrencyCode() === 'CZK') {
            $value = $code->QrPlatbaCz;
        }

        if (empty($value)) {
            $this->logger->error('Paybysquare: empty response');
            return ['success' => false, 'message' => 'empty response'];
        }

        // Return base64
        return ['success' => true, 'message' => 'code generated', 'value' => $value];
    }

    /**
     * @param Order $order
     * @return string
     */
    private function createXML($order)
    {
        $xml =
            '<BySquareXmlDocuments xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">'
            . '<Username>' . $this->data->getUsername() . '</Username>'
            . '<Password>' . $this->data->getPassword() . '</Password>'
            . '<CountryOptions>'
                . '<Slovak>' . ($order->getOrderCurrencyCode() == 'EUR' ? "true" : "false")   . '</Slovak>'
                . '<Czech>' . ($order->getOrderCurrencyCode() === 'CZK' ? "true" : "false")  . '</Czech>'
            . '</CountryOptions>'
            . '<Documents>'
                . '<Pay xsi:type="Pay" xmlns="http://www.bysquare.com/bysquare">'
                    . '<Payments>'
                        . '<Payment>'
                            . '<PaymentOptions>paymentorder</PaymentOptions>'
                            . '<Amount>' . $order->getGrandTotal() . '</Amount>'
                            . '<CurrencyCode>' . $order->getOrderCurrency()->getCurrencyCode() . '</CurrencyCode>'
                            . '<VariableSymbol>' . $order->getIncrementId() . '</VariableSymbol>'
                            . '<PaymentNote>' . 'PAY by square ' . $order->getIncrementId() . '</PaymentNote>'
                            . '<BeneficiaryName>' . $this->data->getBeneficaryName() . '</BeneficiaryName>'
                            . '<BankAccounts>';
                                foreach ($this->data->getBankAccounts($order->getOrderCurrencyCode()) as $bankAccount):
                                    $xml .=
                                        '<BankAccount>'
                                        . '<IBAN>' . $bankAccount['account'] . '</IBAN>'
                                        . '<BIC>' . $bankAccount['bic'] . '</BIC>'
                                        . '</BankAccount>';
                                endforeach;
                                $xml .=
                            '</BankAccounts>'
                        . '</Payment>'
                    . '</Payments>'
                . '</Pay>'
            . '</Documents>'
        . '</BySquareXmlDocuments>';
        return $xml;
    }

    /**
     * Check if payment method is bank transfer
     *
     * @param Order $order
     * @return bool
     */
    private function isBankTranferPayment(Order $order): bool
    {
        return $order->getPayment()->getMethod() === 'banktransfer';
    }

}
